/* jQuery TreeTable Core 2.0 stylesheet
 *
 * This file contains styles that are used to display the tree table. Each tree
 * table is assigned the +treeTable+ class.
 * ========================================================================= */

/* jquery.treeTable.collapsible
 * ------------------------------------------------------------------------- */
.treeTable tr td .expander {
  background-position: left center;
  background-repeat: no-repeat;
  cursor: pointer;
  padding: 0;
  zoom: 1; /* IE7 Hack */
}


.treeTable td.action-checkbox, .treeTable th.action-checkbox-column {
  padding-left: 20px;
}
.treeTable tr.collapsed td .expander {
  background-image: url(toggle-expand-dark.png);
}

.treeTable tr.expanded td .expander {
  background-image: url(toggle-collapse-dark.png);
}

#changelist table.treeTable td.disclosure {
    font-size: 12px;
    text-align: left;
    font-weight: bold;
    padding-left: 19px;
}
#changelist table.treeTable td.disclosure input[type="checkbox"] {
    margin-right: 10px;
}

/* jquery.treeTable.sortable
 * ------------------------------------------------------------------------- */
.treeTable tr.selected, .treeTable tr.accept {
  background-color: #3875d7;
  color: #fff;
}

.treeTable .ui-draggable-dragging {
  border-width: 0;
}

.treeTable tr.selected, .treeTable tr.ghost_row td {
  padding : 5px;
  background-color : #ccc;
  border : dotted 1px #eee;
  font-weight : bold;
  text-align : center;
}

.treeTable tr.collapsed.selected td .expander, .treeTable tr.collapsed.accept td .expander {
  background-image: url(toggle-expand-light.png);
}

.treeTable tr.expanded.selected td .expander, .treeTable tr.expanded.accept td .expander {
  background-image: url(toggle-collapse-light.png);
}

.treeTable .ui-draggable-dragging {
  color: #000;
  z-index: 1;
}